#include "ui_QtVTKRenderWindows.h"
#include <qfiledialog.h>
#include "QtVTKRenderWindows.h"
#include "vtkBoxWidget.h"
#include <vtkRenderer.h>
#include <vtkRenderWindow.h>
#include "vtkProperty.h"
#include "vtkPlanes.h"
#include "vtkImageData.h"
#include "vtkMetaImageReader.h"
#include "vtkSmartVolumeMapper.h"
#include "vtkImageResample.h"
#include "vtkColorTransferFunction.h"
#include "vtkPiecewiseFunction.h"
#include "vtkVolumeProperty.h"
#include "vtkRendererCollection.h"
#include "LVS.cuh"
#include "vtkImageExport.h"
#include "vtkImageImport.h"
#include<vector>
QtVTKRenderWindows::QtVTKRenderWindows(int vtkNotUsed(argc), char *argv[])
{
	this->ui = new Ui_QtVTKRenderWindows;
	this->ui->setupUi(this);

	// Create the renderers and renderwindows.
	vtkSmartPointer<vtkRenderer> renderer1 = vtkRenderer::New();
	renderWindow1 = vtkRenderWindow::New();
	renderWindow1->AddRenderer(renderer1);
	this->ui->view1->SetRenderWindow(renderWindow1);

	vtkSmartPointer<vtkRenderer> renderer2 = vtkRenderer::New();
	renderWindow2 = vtkRenderWindow::New();
	renderWindow2->AddRenderer(renderer2);
	this->ui->view2->SetRenderWindow(renderWindow2);

	// Set up action signals and slots
	connect(this->ui->actionExit, SIGNAL(triggered()), this, SLOT(slotExit()));
	connect(this->ui->action_oeffnen, SIGNAL(triggered()), this, SLOT(openFile()));
	connect(this->ui->windowSpinner, static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
		this, &QtVTKRenderWindows::setWindow);
	connect(this->ui->levelSpinner, static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
		this, &QtVTKRenderWindows::setLevel);
	connect(this->ui->filterBtn, SIGNAL(pressed()), this, SLOT(filterVolume()));
};

vtkSmartPointer<vtkVolume> QtVTKRenderWindows::createVolume(vtkSmartPointer<vtkAlgorithm> reader)
{
	// Create our volume and mapper
	vtkSmartPointer<vtkVolume> volume = vtkVolume::New();
	vtkSmartPointer<vtkSmartVolumeMapper> mapper = vtkSmartVolumeMapper::New();
	mapper->SetInputConnection(reader->GetOutputPort());
	
	// Create the property and attach the transfer functions
	vtkSmartPointer<vtkVolumeProperty> property = vtkVolumeProperty::New();
	property->SetIndependentComponents(true);
	property->SetInterpolationTypeToLinear();

	// connect up the volume to the property and the mapper
	volume->SetProperty(property);
	volume->SetMapper(mapper);

	//Get initial values from GUI
	opacityWindow = ui->windowSpinner->value();
	opacityLevel = ui->levelSpinner->value();

	mapper->SetBlendModeToComposite();
	property->ShadeOff();

	return volume;
}

void QtVTKRenderWindows::filterVolume()
{
	//Get data from GUI.
	int samples = ui->sampleSpinBox->value();
	int angles = ui->anglesSpinBox->value();
	int integration = ui->integrationStepsSpinBox->value();

	//Filter volume.
	LVS* lvs = new LVS();
	float *filteredVoxels = new float[dimensions[0] * dimensions[1] * dimensions[2]];
	lvs->filter(normalisedVoxels, dimensions[0], dimensions[1], dimensions[2], samples, integration, angles, filteredVoxels);

	//Convert normalized float array back to unsigned char.
	FILE *fp = NULL;
	std::vector<unsigned char> outputData;
	outputData.resize(dimensions[0] * dimensions[1] * dimensions[2]);
	for (int i = 0; i < dimensions[0] * dimensions[1] * dimensions[2]; i++) {
		unsigned char reduc = (unsigned char)(255.0f*(filteredVoxels[i]));
		outputData[i] = reduc;
	}

	//Save file
	fopen_s(&fp, "Output.raw", "wb");
	fwrite((void*)&(outputData.front()), sizeof(unsigned char), dimensions[0] * dimensions[1] * dimensions[2], fp);
	fclose(fp);

	//Convert raw back to volume
	vtkSmartPointer<vtkImageImport> importer =
		vtkSmartPointer<vtkImageImport>::New();
	importer->SetDataSpacing(1, 1, 1);
	importer->SetDataOrigin(0, 0, 0);
	importer->SetWholeExtent(0, dimensions[0] - 1, 0, dimensions[1] - 1, 0, dimensions[2] - 1);
	importer->SetDataExtentToWholeExtent();
	importer->SetDataScalarTypeToUnsignedChar();
	importer->SetNumberOfScalarComponents(1);
	importer->SetImportVoidPointer((void*)&outputData.front());
	importer->Update();  
	
	vtkSmartPointer<vtkImageData> input = importer->GetOutput();
	vtkSmartPointer<vtkAlgorithm> reader = importer;
	volume2 = createVolume(reader);
	setTransferFunction();
	//Give volume to renderer
	if (renderWindow2->GetRenderers()->GetFirstRenderer()->GetVolumes()->GetNumberOfItems() > 0) {
		renderWindow2->GetRenderers()->GetFirstRenderer()->RemoveVolume(
			renderWindow2->GetRenderers()->GetFirstRenderer()->GetVolumes()->GetLastProp());
	}
	renderWindow2->GetRenderers()->GetFirstRenderer()->AddVolume(volume2);
	renderWindow2->GetRenderers()->GetFirstRenderer()->ResetCamera();
	renderWindow2->Render();
}

void QtVTKRenderWindows::openFile()
{
	cout << "Open file" << endl;
	QString headerName = QFileDialog::getOpenFileName(this, "Please select Header Data");
	if (headerName != nullptr)
	{
		// Read the data
		vtkSmartPointer<vtkAlgorithm> reader = 0;
		vtkSmartPointer<vtkImageData> input = 0;
		vtkSmartPointer<vtkMetaImageReader> metaReader = vtkMetaImageReader::New(); 
		metaReader->SetFileName(headerName.toStdString().c_str());
		metaReader->Update();
		input = metaReader->GetOutput();
		reader = metaReader;

		// Add the volume to the scene
		if (renderWindow1->GetRenderers()->GetFirstRenderer()->GetVolumes()->GetNumberOfItems() > 0) {
			renderWindow1->GetRenderers()->GetFirstRenderer()->RemoveVolume(
				renderWindow1->GetRenderers()->GetFirstRenderer()->GetVolumes()->GetLastProp());
		}
		if (renderWindow2->GetRenderers()->GetFirstRenderer()->GetVolumes()->GetNumberOfItems() > 0) {
			renderWindow2->GetRenderers()->GetFirstRenderer()->RemoveVolume(
				renderWindow2->GetRenderers()->GetFirstRenderer()->GetVolumes()->GetLastProp());
		}
		volume1 = createVolume(reader);
		setTransferFunction();
		renderWindow1->GetRenderers()->GetFirstRenderer()->AddVolume(volume1);
		renderWindow1->GetRenderers()->GetFirstRenderer()->ResetCamera();
		renderWindow1->Render();
		dimensions = input->GetDimensions();

		//Export raw data from volume for later filtering
		unsigned char *cImage = new unsigned char[dimensions[0] * dimensions[1] * dimensions[2]]; 
		normalisedVoxels = new float[dimensions[0] * dimensions[1] * dimensions[2]];
		vtkSmartPointer<vtkImageExport> exporter =
			vtkSmartPointer<vtkImageExport>::New();
		exporter->SetInputData(input);
		exporter->ImageLowerLeftOn();
		exporter->Update();
		exporter->Export(cImage);
		
		//Normalise raw Data
		for (int i = 0; i < dimensions[0] * dimensions[1] * dimensions[2]; i++)
		{
			const float value = (cImage[i] / (255.0f));
			normalisedVoxels[i] = value;
		}

		//Ready to filter
		ui->filterBtn->setEnabled(true);
	}
}

void QtVTKRenderWindows::setLevel()
{
	opacityLevel = this->ui->levelSpinner->value();
	// Create our transfer function
	setTransferFunction();
	renderWindow1->Render();
	renderWindow2->Render();
}

void QtVTKRenderWindows::setWindow()
{
	opacityWindow = this->ui->windowSpinner->value();
	// Create our transfer function
	setTransferFunction();
	renderWindow1->Render();
	renderWindow2->Render();
}

void QtVTKRenderWindows::setTransferFunction() {
	vtkSmartPointer<vtkColorTransferFunction> colorFun = vtkColorTransferFunction::New();
	vtkSmartPointer<vtkPiecewiseFunction> opacityFun = vtkPiecewiseFunction::New();
	if (volume1 != nullptr) {
		volume1->GetProperty()->SetColor(colorFun);
		volume1->GetProperty()->SetScalarOpacity(opacityFun);
	}
	if (volume2 != nullptr) {
		volume2->GetProperty()->SetColor(colorFun);
		volume2->GetProperty()->SetScalarOpacity(opacityFun);
	}
	colorFun->AddRGBSegment(opacityLevel - 0.5*opacityWindow, 0.0, 0.0, 0.0,
		opacityLevel + 0.5*opacityWindow, 1.0, 1.0, 1.0);
	opacityFun->AddSegment(opacityLevel - 0.5*opacityWindow, 0.0,
		opacityLevel + 0.5*opacityWindow, 1.0);
}

void QtVTKRenderWindows::slotExit()
{
	qApp->exit();
}

void QtVTKRenderWindows::Render()
{
	this->ui->view1->GetRenderWindow()->Render();
	this->ui->view2->GetRenderWindow()->Render();
}

