#ifndef QtVTKRenderWindows_H
#define QtVTKRenderWindows_H

#include "vtkSmartPointer.h"
#include "vtkVolume.h"
#include "vtkResliceImageViewer.h"
#include "vtkImagePlaneWidget.h"
#include "vtkDistanceWidget.h"
#include "vtkResliceImageViewerMeasurements.h"
#include <QMainWindow>
// Forward Qt class declarations
class Ui_QtVTKRenderWindows;

/*!
* Implementation of a simple VTK powered volume renderer using QT as GUI.
*
*/
class QtVTKRenderWindows : public QMainWindow
{
	Q_OBJECT
public:

	// Constructor/Destructor
	QtVTKRenderWindows(int argc, char *argv[]);
	~QtVTKRenderWindows() {}

	public slots:

	virtual void slotExit();
	virtual void Render();
	virtual void openFile();
	/*!
	* Exports raw output data, gives it to filter, converts it back and saves it. 
	* Gives resulting filtered volume to renderwindow2.
	* @param[in] reader VtkAlgorithm that holds the volume data.
	*/
	virtual void filterVolume();
	/*!
	* Sets opacitylevel when changed in GUI.
	*/
	virtual void setLevel();
	/*!
	* Sets opacitywindow when changed in GUI.
	*/
	virtual void setWindow();

protected:
	/*!
	* RenderWindow for the unfiltered volume.
	*/
	vtkSmartPointer<vtkRenderWindow> renderWindow1;
	/*!
	* RenderWindow for the filtered volume.
	*/
	vtkSmartPointer<vtkRenderWindow> renderWindow2;
	/*!
	* The unfiltered volume.
	*/
	vtkSmartPointer<vtkVolume> volume1;
	/*!
	* The filtered volume.
	*/
	vtkSmartPointer<vtkVolume> volume2;
	/*!
	* Flat array for filterinput.
	*/
	float* normalisedVoxels;
	int* dimensions;
	/*!
	* Used in ramp function.
	*/
	double opacityLevel;
	/*!
	* Used in ramp function.
	*/
	double opacityWindow;
	protected slots:

private:
	/*!
	* Returns a volume with mapper and transfer function from the given input.
	* @param[in] reader VtkAlgorithm that holds the volume data.
	*/
	vtkSmartPointer<vtkVolume> createVolume(vtkSmartPointer<vtkAlgorithm> reader);
	void setTransferFunction();
	// Designer form
	Ui_QtVTKRenderWindows *ui;
};

#endif // QtVTKRenderWindows_H
